@@ -270,14 +270,9 @@ __version__ = "$Revision$"[11:-2]
 __docformat__ = "restructuredtext"
 
 
-import datetime, locale, md5, site, sys, time, urllib, urllib2, ConfigParser, os
+import datetime, locale, hashlib, site, sys, time, urllib, urllib2, ConfigParser, os
 
-try:
-	# Python 2.5, module bundled:
-	from xml.etree import ElementTree
-except:
-	# Python 2.4, separate module:
-	from elementtree import ElementTree
+from xml.etree import ElementTree
 
 # This is lifted in the most part from iPodScrobbler (see docs above)
 # Get the base local encoding
@@ -300,7 +295,7 @@ audioscrobbler_request_host = 'ws.audioscrobbler.com'
 # AudioScrobblerPost configuration settings
 audioscrobbler_post_version = u'1.2' #updated
 audioscrobbler_post_host = 'post.audioscrobbler.com'
-client_name = u'tst'
+client_name = u'sna'
 pyscrobbler_version = u'1.0.0.0' # This is set to 1.0 while we use
 							 # client_name = u'tst' as we keep getting
 							 # UPDATE responses with anything less.
@@ -414,12 +409,12 @@ class AudioScrobbler:
 								  **kwargs)
 		return ret
 		
-	def post(self, username, password, verbose=False):
+	def post(self, username, md5_password, verbose=False):
 		
 		""" Create a new AudioScrobblerPost """
 		
 		ret = AudioScrobblerPost(username=username.encode('utf8'),
-								 password=password.encode('utf8'),
+								 md5_password=md5_password.encode('utf8'),
 								 host=self.audioscrobbler_post_host,
 								 protocol_version=self.audioscrobbler_post_version,
 								 client_name=self.client_name,
@@ -583,7 +578,7 @@ class AudioScrobblerPost:
 	
 	def __init__(self,
 				 username=u'',
-				 password=u'',
+				 md5_password=u'',
 				 client_name=client_name,
 				 client_version=pyscrobbler_version,
 				 protocol_version=audioscrobbler_post_version,
@@ -592,7 +587,7 @@ class AudioScrobblerPost:
 	
 		# Capture the information passed for future use
 		self.params = dict(username=username,
-						   password=password,
+						   md5_password=md5_password,
 						   client_name=client_name,
 						   client_version=client_version,
 						   protocol_version=protocol_version,
@@ -619,8 +614,8 @@ class AudioScrobblerPost:
 			return True
 		
 		timestamp = str(int(time.time()))
-		password = self.params['password']
-		auth_token = md5.md5(md5.md5(password).hexdigest() + timestamp).hexdigest()
+		md5_password = self.params['md5_password']
+		auth_token = hashlib.md5(md5_password + timestamp).hexdigest()
 		
 		p = {}
 		p['hs'] = 'true'
@@ -652,7 +647,7 @@ class AudioScrobblerPost:
 			raise AudioScrobblerHandshakeError('Got nothing back from the server')	
 			
 		username = self.params['username']
-		password = self.params['password']
+		md5_password = self.params['md5_password']
 		
 		# First we test the best and most likely case
 		if response[0].startswith('OK'):
@@ -746,15 +741,15 @@ class AudioScrobblerPost:
 		# Otherwise build the track dictionary and add it to the local cache
 		else:
 			try:
-				track = {'a[%s]': artist_name.decode(enc).encode('utf8'),
-					 	't[%s]': song_title.decode(enc).encode('utf8'),
+				track = {'a[%s]': artist_name.encode('utf8'),
+					 	't[%s]': song_title.encode('utf8'),
 					 	'l[%s]': str(sane_length),
 					 	'i[%s]': date_played,
-					 	'b[%s]': album.decode(enc).encode('utf8'),
+					 	'b[%s]': album.encode('utf8'),
 					 	'm[%s]': mbid.encode('utf8'),
 					 	'r[%s]': u'',
-					 	'n[%s]': tracknumber.decode(enc).encode('utf8'),
-					 	'o[%s]': source.decode(enc).encode('utf8'),
+					 	'n[%s]': tracknumber.encode('utf8'),
+					 	'o[%s]': source.encode('utf8'),
 						}
 			except:
 				track = {'a[%s]': artist_name,
@@ -781,11 +776,10 @@ class AudioScrobblerPost:
 		
 		p = {}
 		p['s'] = self.auth_details['s']
-		
 		try:
-			p['a'] = artist_name.decode(enc).encode('utf8')
-			p['t'] = song_title.decode(enc).encode('utf8')
-			p['b'] = album.decode(enc).encode('utf8')
+			p['a'] = artist_name.encode('utf8')
+			p['t'] = song_title.encode('utf8')
+			p['b'] = album.encode('utf8')
 			p['l'] = length.encode('utf8')
 			p['n'] = tracknumber.encode('utf8')
 			p['m'] = mbid.encode('utf8')
