@@ -1,15 +1,13 @@
 
 import sys, os, locale, urllib
 import threading # get_lyrics_start starts a thread get_lyrics_thread
-from socket import getdefaulttimeout as socketgettimeout
-from socket import setdefaulttimeout as socketsettimeout
 
 import gtk, gobject
-ServiceProxy = None # importing tried when needed
 
 import ui, misc
 import mpdhelper as mpdh
 from consts import consts
+import re
 
 class Info(object):
 	def __init__(self, config, info_image, linkcolor, on_link_click_cb, library_return_search_items, get_playing_song, TAB_INFO, on_image_activate, on_image_motion_cb, on_image_drop_cb, album_return_artist_and_tracks, new_tab):
@@ -30,7 +28,6 @@ class Info(object):
 			print "Locale cannot be found; please set your system's locale. Aborting..."
 			sys.exit(1)
 
-		self.lyricServer = None
 		self.last_info_bitrate = None
 
 		self.info_boxes_in_more = None
@@ -87,35 +84,36 @@ class Info(object):
 		self.info_type = {}
 		self.info_labels = []
 		self.info_boxes_in_more = []
-		labels_type = ['title', 'artist', 'album', 'date', 'track', 'genre', 'file', 'bitrate']
-		labels_text = [_("Title"), _("Artist"), _("Album"), _("Date"), _("Track"), _("Genre"), _("File"), _("Bitrate")]
-		labels_link = [False, True, True, False, False, False, False, False]
-		labels_tooltip = ["", _("Launch artist in Wikipedia"), _("Launch album in Wikipedia"), "", "", "", "", ""]
-		labels_in_more = [False, False, False, False, False, False, True, True]
-		for i in range(len(labels_text)):
-			self.info_type[labels_text[i]] = i
+		labels = [(_("Title"), 'title', False, "", False),
+			(_("Artist"), 'artist', True,
+				_("Launch artist in Wikipedia"), False),
+			(_("Album"), 'album', True,
+				 _("Launch album in Wikipedia"), False),
+			(_("Date"), 'date', False, "", False),
+			(_("Track"), 'track', False, "", False),
+			(_("Genre"), 'genre', False, "", False),
+			(_("File"), 'file', False, "", True),
+			(_("Bitrate"), 'bitrate', False, "", True)]
+
+		for i,(text, name, link, tooltip, in_more) in enumerate(labels):
+			self.info_type[name] = i
 			tmphbox = gtk.HBox()
-			if labels_in_more[i]:
+			if in_more:
 				self.info_boxes_in_more += [tmphbox]
-			tmplabel = ui.label(markup="<b>" + labels_text[i] + ":</b>", y=0)
+			tmplabel = ui.label(markup="<b>%s:</b>" % text, y=0)
 			if i == 0:
 				self.info_left_label = tmplabel
-			if not labels_link[i]:
-				tmplabel2 = ui.label(wrap=True, y=0, select=True)
-			else:
-				# Using set_selectable overrides the hover cursor that sonata
-				# tries to set for the links, and I can't figure out how to
-				# stop that. So we'll disable set_selectable for these two
-				# labels until it's figured out.
-				tmplabel2 = ui.label(wrap=True, y=0, select=False)
-			if labels_link[i]:
+			# Using set_selectable overrides the hover cursor that
+			# sonata tries to set for the links, and I can't figure
+			# out how to stop that. So we'll disable set_selectable
+			# for those labels until it's figured out.
+			tmplabel2 = ui.label(wrap=True, y=0, select=not link)
+			if link:
 				tmpevbox = ui.eventbox(add=tmplabel2)
-				self.info_apply_link_signals(tmpevbox, labels_type[i], labels_tooltip[i])
+				self.info_apply_link_signals(tmpevbox, name, tooltip)
 			tmphbox.pack_start(tmplabel, False, False, horiz_spacing)
-			if labels_link[i]:
-				tmphbox.pack_start(tmpevbox, False, False, horiz_spacing)
-			else:
-				tmphbox.pack_start(tmplabel2, False, False, horiz_spacing)
+			to_pack = tmpevbox if link else tmplabel2
+			tmphbox.pack_start(to_pack, False, False, horiz_spacing)
 			self.info_labels += [tmplabel2]
 			labels_left += [tmplabel]
 			self.info_tagbox.pack_start(tmphbox, False, False, vert_spacing)
@@ -241,14 +239,14 @@ class Info(object):
 		# want to update the minimum number of widgets so the user can
 		# do things like select label text.
 		if playing_or_paused:
-			bitratelabel = self.info_labels[self.info_type[_("Bitrate")]]
-			titlelabel = self.info_labels[self.info_type[_("Title")]]
-			artistlabel = self.info_labels[self.info_type[_("Artist")]]
-			albumlabel = self.info_labels[self.info_type[_("Album")]]
-			datelabel = self.info_labels[self.info_type[_("Date")]]
-			genrelabel = self.info_labels[self.info_type[_("Genre")]]
-			tracklabel = self.info_labels[self.info_type[_("Track")]]
-			filelabel = self.info_labels[self.info_type[_("File")]]
+			bitratelabel = self.info_labels[self.info_type['bitrate']]
+			titlelabel = self.info_labels[self.info_type['title']]
+			artistlabel = self.info_labels[self.info_type['artist']]
+			albumlabel = self.info_labels[self.info_type['album']]
+			datelabel = self.info_labels[self.info_type['date']]
+			genrelabel = self.info_labels[self.info_type['genre']]
+			tracklabel = self.info_labels[self.info_type['track']]
+			filelabel = self.info_labels[self.info_type['file']]
 			if not self.last_info_bitrate or self.last_info_bitrate != newbitrate:
 				bitratelabel.set_text(newbitrate)
 			self.last_info_bitrate = newbitrate
@@ -292,6 +290,7 @@ class Info(object):
 						albuminfo = album + "\n"
 					playtime = 0
 					if len(tracks) > 0:
+						tracks.sort(key=lambda x: int(mpdh.getnum(x, 'track', '0', True, 0)))
 						for track in tracks:
 							playtime += int(mpdh.get(track, 'time', 0))
 							if 'title' in track:
@@ -311,18 +310,7 @@ class Info(object):
 					self.albumText.set_text(_("Album name not set."))
 				# Update lyrics:
 				if self.config.show_lyrics and not skip_lyrics:
-					global ServiceProxy
-					if ServiceProxy is None:
-						try:
-							from ZSI import ServiceProxy
-							# Make sure we have the right version..
-							_test = ServiceProxy.ServiceProxy
-						except:
-							ServiceProxy = None
-					if ServiceProxy is None:
-						self.info_searchlabel.set_text("")
-						self.info_show_lyrics(_("ZSI not found, fetching lyrics support disabled."), "", "", True)
-					elif 'artist' in songinfo and 'title' in songinfo:
+					if 'artist' in songinfo and 'title' in songinfo:
 						self.get_lyrics_start(mpdh.get(songinfo, 'artist'), mpdh.get(songinfo, 'title'), mpdh.get(songinfo, 'artist'), mpdh.get(songinfo, 'title'), os.path.dirname(mpdh.get(songinfo, 'file')))
 					else:
 						self.info_searchlabel.set_text("")
@@ -341,15 +329,15 @@ class Info(object):
 			self.last_info_bitrate = ""
 
 	def info_check_for_local_lyrics(self, artist, title, song_dir):
-		if os.path.exists(self.target_lyrics_filename(artist, title, song_dir, consts.LYRICS_LOCATION_HOME)):
-			return self.target_lyrics_filename(artist, title, song_dir, consts.LYRICS_LOCATION_HOME)
-		elif os.path.exists(self.target_lyrics_filename(artist, title, song_dir, consts.LYRICS_LOCATION_PATH)):
-			return self.target_lyrics_filename(artist, title, song_dir, consts.LYRICS_LOCATION_PATH)
-		elif os.path.exists(self.target_lyrics_filename(artist, title, song_dir, consts.LYRICS_LOCATION_HOME_ALT)):
-			return self.target_lyrics_filename(artist, title, song_dir, consts.LYRICS_LOCATION_HOME_ALT)
-		elif os.path.exists(self.target_lyrics_filename(artist, title, song_dir, consts.LYRICS_LOCATION_PATH_ALT)):
-			return self.target_lyrics_filename(artist, title, song_dir, consts.LYRICS_LOCATION_PATH_ALT)
-		return None
+		locations = [consts.LYRICS_LOCATION_HOME,
+				consts.LYRICS_LOCATION_PATH,
+				consts.LYRICS_LOCATION_HOME_ALT,
+				consts.LYRICS_LOCATION_PATH_ALT]
+		for location in locations:
+			filename = self.target_lyrics_filename(artist, title,
+							    song_dir, location)
+			if os.path.exists(filename):
+				return filename
 
 	def get_lyrics_start(self, *args):
 		lyricThread = threading.Thread(target=self.get_lyrics_thread, args=args)
@@ -399,48 +387,30 @@ class Info(object):
 			filename = self.target_lyrics_filename(filename_artist, filename_title, song_dir)
 			# Fetch lyrics from lyricwiki.org
 			gobject.idle_add(self.info_show_lyrics, _("Fetching lyrics..."), filename_artist, filename_title)
-			if self.lyricServer is None:
-				wsdlFile = "http://lyricwiki.org/server.php?wsdl"
-				try:
-					self.lyricServer = True
-					timeout = socketgettimeout()
-					socketsettimeout(consts.LYRIC_TIMEOUT)
-					self.lyricServer = ServiceProxy.ServiceProxy(wsdlFile, cachedir=os.path.expanduser("~/.service_proxy_dir")) 
-				except:
-					socketsettimeout(timeout)
-					lyrics = _("Couldn't connect to LyricWiki")
-					gobject.idle_add(self.info_show_lyrics, lyrics, filename_artist, filename_title)
-					self.lyricServer = None
-					gobject.idle_add(self.info_searchlabel.set_markup, search_str)
-					gobject.idle_add(self.info_editlyricslabel.set_markup, edit_str)
-					return
 			try:
-				timeout = socketgettimeout()
-				socketsettimeout(consts.LYRIC_TIMEOUT)
-				lyrics = self.lyricServer.getSong(artist=self.lyricwiki_format(search_artist), song=self.lyricwiki_format(search_title))['return']["lyrics"]
-				if lyrics.lower() != "not found":
+				lyricpage = urllib.urlopen("http://lyricwiki.org/index.php?title=%s:%s&action=edit" % (self.lyricwiki_format(search_artist), self.lyricwiki_format(search_title))).read()
+				content = re.split("<textarea[^>]*>", lyricpage)[1].split("</textarea>")[0]
+				if content.startswith("#REDIRECT [["):
+					addr = "http://lyricwiki.org/index.php?title=%s&action=edit" % urllib.quote(content.split("[[")[1].split("]]")[0])
+					content = urllib.urlopen(addr).read()
+				lyrics = content.split("&lt;lyrics&gt;")[1].split("&lt;/lyrics&gt;")[0]
+				if lyrics.strip() != "&lt;!-- PUT LYRICS HERE (and delete this entire line) --&gt;":
 					lyrics = misc.unescape_html(lyrics)
 					lyrics = misc.wiki_to_html(lyrics)
-					lyrics = lyrics.encode("ISO-8859-1")
-					gobject.idle_add(self.info_show_lyrics, lyrics, filename_artist, filename_title)
-					# Save lyrics to file:
+					lyrics = lyrics.decode("utf-8")
+# Save lyrics to file:
 					misc.create_dir('~/.lyrics/')
 					f = open(filename, 'w')
-					lyrics = misc.unescape_html(lyrics)
-					try:
-						f.write(lyrics.decode(self.enc).encode('utf8'))
-					except:
-						f.write(lyrics)
+					f.write(lyrics)
 					f.close()
 				else:
 					lyrics = _("Lyrics not found")
-					gobject.idle_add(self.info_show_lyrics, lyrics, filename_artist, filename_title)
+				gobject.idle_add(self.info_show_lyrics, lyrics, filename_artist, filename_title)
 			except:
 				lyrics = _("Fetching lyrics failed")
 				gobject.idle_add(self.info_show_lyrics, lyrics, filename_artist, filename_title)
 			gobject.idle_add(self.info_searchlabel.set_markup, search_str)
 			gobject.idle_add(self.info_editlyricslabel.set_markup, edit_str)
-			socketsettimeout(timeout)
 
 	def info_show_lyrics(self, lyrics, artist, title, force=False):
 		if force:
@@ -475,19 +445,16 @@ class Info(object):
 
 	def target_lyrics_filename(self, artist, title, song_dir, force_location=None):
 		# FIXME Why did we have this condition here: if self.conn:
-			if force_location is not None:
-				lyrics_loc = force_location
-			else:
-				lyrics_loc = self.config.lyrics_location
-			# Note: *_ALT searching is for compatibility with other mpd clients (like ncmpcpp):
-			if lyrics_loc == consts.LYRICS_LOCATION_HOME:
-				targetfile = os.path.expanduser("~/.lyrics/" + artist + "-" + title + ".txt")
-			elif lyrics_loc == consts.LYRICS_LOCATION_PATH:
-				targetfile = self.config.musicdir[self.config.profile_num] + song_dir + "/" + artist + "-" + title + ".txt"
-			elif lyrics_loc == consts.LYRICS_LOCATION_HOME_ALT:
-				targetfile = os.path.expanduser("~/.lyrics/" + artist + " - " + title + ".txt")
-			elif lyrics_loc == consts.LYRICS_LOCATION_PATH_ALT:
-				targetfile = self.config.musicdir[self.config.profile_num] + song_dir + "/" + artist + " - " + title + ".txt"
-			targetfile = misc.file_exists_insensitive(targetfile)
-			return misc.file_from_utf8(targetfile)
+		lyrics_loc = force_location if force_location else self.config.lyrics_location
+		# Note: *_ALT searching is for compatibility with other mpd clients (like ncmpcpp):
+		if lyrics_loc == consts.LYRICS_LOCATION_HOME:
+			targetfile = os.path.expanduser("~/.lyrics/" + artist + "-" + title + ".txt")
+		elif lyrics_loc == consts.LYRICS_LOCATION_PATH:
+			targetfile = self.config.musicdir[self.config.profile_num] + song_dir + "/" + artist + "-" + title + ".txt"
+		elif lyrics_loc == consts.LYRICS_LOCATION_HOME_ALT:
+			targetfile = os.path.expanduser("~/.lyrics/" + artist + " - " + title + ".txt")
+		elif lyrics_loc == consts.LYRICS_LOCATION_PATH_ALT:
+			targetfile = self.config.musicdir[self.config.profile_num] + song_dir + "/" + artist + " - " + title + ".txt"
+		targetfile = misc.file_exists_insensitive(targetfile)
+		return misc.file_from_utf8(targetfile)
 	
