@@ -8,6 +8,7 @@ import gtk, gobject, pango
 import ui, misc
 import mpdhelper as mpdh
 from consts import consts
+import breadcrumbs
 
 def library_set_data(album=None, artist=None, genre=None, year=None, path=None):
 	return (album, artist, genre, year, path)
@@ -16,12 +17,7 @@ def library_get_data(data, *args):
 	name_to_index = {'album':0, 'artist':1, 'genre':2, 'year':3, 'path':4}
 	# Data retrieved from the gtktreeview model is not in
 	# unicode anymore, so convert it.
-	retlist = []
-	for arg in list(args):
-		ret = data[name_to_index[arg]]
-		if ret is not None:
-			ret = unicode(ret)
-		retlist.append(ret)
+	retlist = [unicode(data[name_to_index[arg]]) if data[name_to_index[arg]] else None for arg in args]
 	if len(retlist) == 1:
 		return retlist[0]
 	else:
@@ -49,11 +45,6 @@ class Library(object):
 		self.search_terms = [_('Artist'), _('Title'), _('Album'), _('Genre'), _('Filename'), _('Everything')]
 		self.search_terms_mpd = ['artist', 'title', 'album', 'genre', 'file', 'any']
 
-		self.VIEW_FILESYSTEM_STR = _("Filesystem")
-		self.VIEW_ARTIST_STR = _("Artists")
-		self.VIEW_ALBUM_STR = _("Albums")
-		self.VIEW_GENRE_STR = _("Genres")
-
 		self.libfilterbox_cmd_buf = None
 		self.libfilterbox_cond = None
 		self.libfilterbox_source = None
@@ -78,6 +69,8 @@ class Library(object):
 		self.libraryvbox = gtk.VBox()
 		self.library = ui.treeview()
 		self.library_selection = self.library.get_selection()
+		self.breadcrumbs = breadcrumbs.CrumbBox()
+		self.breadcrumbs.props.spacing = 2
 		expanderwindow2 = ui.scrollwindow(add=self.library)
 		self.searchbox = gtk.HBox()
 		self.searchcombo = ui.combo(items=self.search_terms)
@@ -90,26 +83,43 @@ class Library(object):
 		self.searchbutton.set_tooltip_text(_("End Search"))
 		self.libraryview = ui.button(relief=gtk.RELIEF_NONE)
 		self.libraryview.set_tooltip_text(_("Library browsing view"))
-		self.library_view_assign_image()
+		# disabled as breadcrumbs replace this:
 		self.searchbox.pack_start(self.libraryview, False, False, 1)
 		self.searchbox.pack_start(gtk.VSeparator(), False, False, 2)
 		self.searchbox.pack_start(ui.label(_("Search") + ":"), False, False, 3)
 		self.searchbox.pack_start(self.searchtext, True, True, 2)
 		self.searchbox.pack_start(self.searchcombo, False, False, 2)
 		self.searchbox.pack_start(self.searchbutton, False, False, 2)
+		#self.libraryvbox.pack_start(self.breadcrumbs, False, False, 2)
 		self.libraryvbox.pack_start(expanderwindow2, True, True)
 		self.libraryvbox.pack_start(self.searchbox, False, False, 2)
 
 		self.tab = new_tab(self.libraryvbox, gtk.STOCK_HARDDISK, TAB_LIBRARY, self.library)
 
 		# Assign some pixbufs for use in self.library
-		self.openpb = self.library.render_icon(gtk.STOCK_OPEN, gtk.ICON_SIZE_LARGE_TOOLBAR)
-		self.harddiskpb = self.library.render_icon(gtk.STOCK_HARDDISK, gtk.ICON_SIZE_LARGE_TOOLBAR)
+		self.openpb2 = self.library.render_icon(gtk.STOCK_OPEN, gtk.ICON_SIZE_LARGE_TOOLBAR)
+		self.harddiskpb2 = self.library.render_icon(gtk.STOCK_HARDDISK, gtk.ICON_SIZE_LARGE_TOOLBAR)
+		self.openpb = self.library.render_icon(gtk.STOCK_OPEN, gtk.ICON_SIZE_MENU)
+		self.harddiskpb = self.library.render_icon(gtk.STOCK_HARDDISK, gtk.ICON_SIZE_MENU)
 		self.albumpb = gtk.gdk.pixbuf_new_from_file_at_size(album_filename, consts.LIB_COVER_SIZE, consts.LIB_COVER_SIZE)
 		self.genrepb = self.library.render_icon('gtk-orientation-portrait', gtk.ICON_SIZE_LARGE_TOOLBAR)
 		self.artistpb = self.library.render_icon('artist', gtk.ICON_SIZE_LARGE_TOOLBAR)
 		self.sonatapb = self.library.render_icon('sonata', gtk.ICON_SIZE_MENU)
 
+		# list of the library views: (id, name, icon name, label)
+		self.VIEWS = [
+			(consts.VIEW_FILESYSTEM, 'filesystem', 
+			 gtk.STOCK_HARDDISK, _("Filesystem")),
+			(consts.VIEW_ALBUM, 'album', 
+			 'album', _("Albums")),
+			(consts.VIEW_ARTIST, 'artist', 
+			 'artist', _("Artists")),
+			(consts.VIEW_GENRE, 'genre', 
+			 gtk.STOCK_ORIENTATION_PORTRAIT, _("Genres")),
+			]
+
+		self.library_view_assign_image()
+
 		self.library.connect('row_activated', self.on_library_row_activated)
 		self.library.connect('button_press_event', self.on_library_button_press)
 		self.library.connect('key-press-event', self.on_library_key_press)
@@ -146,12 +156,9 @@ class Library(object):
 		self.library_selection.set_mode(gtk.SELECTION_MULTIPLE)
 
 	def get_libraryactions(self):
-		return [
-			('filesystemview', gtk.STOCK_HARDDISK, self.VIEW_FILESYSTEM_STR, None, None, self.on_libraryview_chosen),
-			('artistview', 'artist', self.VIEW_ARTIST_STR, None, None, self.on_libraryview_chosen),
-			('genreview', gtk.STOCK_ORIENTATION_PORTRAIT, self.VIEW_GENRE_STR, None, None, self.on_libraryview_chosen),
-			('albumview', 'album', self.VIEW_ALBUM_STR, None, None, self.on_libraryview_chosen), 
-			]
+		return [(name+'view', icon, label,
+			 None, None, self.on_libraryview_chosen)
+			for _view, name, icon, label in self.VIEWS]
 
 	def get_model(self):
 		return self.librarydata
@@ -169,11 +176,11 @@ class Library(object):
 		self.librarymenu = librarymenu
 		self.librarymenu.attach_to_widget(self.libraryview, None)
 
-	def library_view_popup(self, _button):
-		self.librarymenu.popup(None, None, self.library_view_position_menu, 1, 0)
+	def library_view_popup(self, button):
+		self.librarymenu.popup(None, None, self.library_view_position_menu, 1, 0, button)
 
-	def library_view_position_menu(self, _menu):
-		x, y, _width, height = self.libraryview.get_allocation()
+	def library_view_position_menu(self, _menu, button):
+		x, y, _width, height = button.get_allocation()
 		return (self.config.x + x, self.config.y + y + height, True)
 
 	def on_libraryview_chosen(self, action):
@@ -200,15 +207,10 @@ class Library(object):
 		gobject.idle_add(self.library.scroll_to_point, 0, 0)
 	
 	def library_view_assign_image(self):
-		info = [(self.VIEW_FILESYSTEM_STR, gtk.STOCK_HARDDISK, consts.VIEW_FILESYSTEM), \
-		        (self.VIEW_ARTIST_STR, 'artist', consts.VIEW_ARTIST), \
-				(self.VIEW_GENRE_STR, 'gtk-orientation-portrait', consts.VIEW_GENRE), \
-				(self.VIEW_ALBUM_STR, 'album', consts.VIEW_ALBUM)]
-		for i, txt in enumerate(info):
-			if self.config.lib_view == txt[2]:
-				self.libraryview.set_image(ui.image(stock=txt[1]))
-				self.libraryview.set_label(" " + txt[0])
-				break
+		_view, _name, icon, label = [v for v in self.VIEWS 
+					     if v[0] == self.config.lib_view][0]
+		self.libraryview.set_image(ui.image(stock=icon))
+		self.libraryview.set_label(" " + label)
 
 	def view_caches_reset(self):
 		# We should call this on first load and whenever mpd is
@@ -365,9 +367,107 @@ class Library(object):
 		# Update library artwork as necessary
 		self.on_library_scrolled(None, None)
 
+		#self.update_breadcrumbs()
+
+	def update_breadcrumbs(self):
+		# remove previous buttons
+		for b in self.breadcrumbs:
+			self.breadcrumbs.remove(b)
+
+		# add the views button first
+		b = ui.button(text=_(" v "), can_focus=False, relief=gtk.RELIEF_NONE)
+		b.connect('clicked', self.library_view_popup)
+		self.breadcrumbs.pack_start(b, False, False)
+		b.show()
+		
+		# add the ellipsis explicitly XXX make this unnecessary
+		b = ui.label("...")
+		self.breadcrumbs.pack_start(b, False, False)
+		b.show()
+
+		# find info for current view		
+		view, _name, icon, label = [v for v in self.VIEWS 
+						  if v[0] == self.config.lib_view][0]
+
+		# the first crumb is the root of the current view
+		crumbs = [(label, icon, None, self.library_set_data(path='/'))]
+
+		# rest of the crumbs are specific to the view
+		if view == consts.VIEW_FILESYSTEM:
+			path = self.library_get_data(self.config.wd, 'path')
+			
+			if path and path != '/':
+				parts = path.split('/')
+			else:
+				parts = [] # no crumbs for /
+			# append a crumb for each part
+			for i, part in enumerate(parts):
+				partpath = '/'.join(parts[:i+1])
+				target = self.library_set_data(path=partpath)
+				crumbs.append((part, gtk.STOCK_OPEN, None, target))
+		else:
+			if view == consts.VIEW_ALBUM:
+				# We don't want to show an artist button in album view
+				keys = 'genre', 'album'
+				nkeys = 2
+			else:
+				keys = 'genre', 'artist', 'album'
+				nkeys = 3
+			parts = self.library_get_data(self.config.wd, *keys)
+			# append a crumb for each part
+			for i, key, part in zip(range(nkeys), keys, parts):
+				if part is None:
+					continue
+				partdata = dict(zip(keys, parts)[:i+1])
+				target = self.library_set_data(**partdata)
+				pb, icon = None, None
+				if key == 'album':
+					# Album artwork, with self.alumbpb as a backup:
+					artist, album, path = self.library_get_data(self.config.wd, 'artist', 'album', 'path')
+					cache_data = self.library_set_data(artist=artist, album=album, path=path)
+					pb = self.artwork.get_library_artwork_cached_pb(cache_data, None)
+					if pb is None:
+						icon = 'album'
+				elif key == 'artist':
+					icon = 'artist'
+				else:
+					icon = gtk.STOCK_ORIENTATION_PORTRAIT
+				crumbs.append((part, icon, pb, target))
+
+		# add a button for each crumb
+		for crumb in crumbs:
+			text, icon, pb, target = crumb
+			text = misc.escape_html(text)
+			if crumb is crumbs[-1]:
+				text = "<b>%s</b>" % text
+			label = ui.label(markup=text)
+
+			if icon:
+				image = ui.image(stock=icon)
+			elif pb:
+				pb = pb.scale_simple(16, 16, gtk.gdk.INTERP_HYPER)
+				image = ui.image(pb=pb)
+
+			b = breadcrumbs.CrumbButton(image, label)
+
+			if crumb is crumbs[-1]:
+				# FIXME makes the button request minimal space:
+				# label.props.ellipsize = pango.ELLIPSIZE_END
+				b.props.active = True
+			# FIXME why doesn't the tooltip show?
+			b.set_tooltip_text(label.get_label())
+			b.connect('toggled', self.library_browse, target)
+			self.breadcrumbs.pack_start(b, False, False)
+			b.show_all()
+
 	def library_populate_add_parent_rows(self):
-		bd = [('0', [self.harddiskpb, self.library_set_data(path='/'), '/'])]
-		bd += [('1', [self.openpb, self.library_set_data(path='..'), '..'])]
+		#return [] # disabled as breadcrumbs replace these
+		if self.config.lib_view == consts.VIEW_FILESYSTEM:
+			bd = [('0', [self.harddiskpb, self.library_set_data(path='/'), '/'])]
+			bd += [('1', [self.openpb, self.library_set_data(path='..'), '..'])]
+		else:
+			bd = [('0', [self.harddiskpb2, self.library_set_data(path='/'), '/'])]
+			bd += [('1', [self.openpb2, self.library_set_data(path='..'), '..'])]
 		return bd
 
 	def library_populate_filesystem_data(self, path):
@@ -489,6 +589,10 @@ class Library(object):
 				if unicode(self.library_get_data(albums[i], 'album')).lower() != unicode(self.library_get_data(albums[i+j], 'album')).lower() \
 				or self.library_get_data(albums[i], 'year') != self.library_get_data(albums[i+j], 'year'):
 					break
+				if unicode(self.library_get_data(albums[i], 'artist')) == unicode(self.library_get_data(albums[i+j], 'artist')):
+					albums.pop(i+j)
+					break
+
 				if j == consts.NUM_ARTISTS_FOR_VA - 1:
 					VA = True
 			if VA:
@@ -499,6 +603,7 @@ class Library(object):
 				while i+j <= len(albums)-1:
 					if unicode(self.library_get_data(albums[i], 'album')).lower() == unicode(self.library_get_data(albums[i+j], 'album')).lower() \
 					and self.library_get_data(albums[i], 'year') == self.library_get_data(albums[i+j], 'year'):
+
 						albums.pop(i+j)
 					else:						
 						break
@@ -766,9 +871,9 @@ class Library(object):
 			self.library_selection.unselect_range((0,), (len(self.librarydata)-1,))
 		# Now attempt to retain the selection from before the update:
 		for value in prev_selection:
-			for rownum in range(len(self.librarydata)):
-				if value == self.librarydata.get_value(self.librarydata.get_iter((rownum,)), 1):
-					self.library_selection.select_path((rownum,))
+			for row in self.librarydata:
+				if value == row[1]:
+					self.library_selection.select_path(row.path)
 					break
 		if prev_selection_root:
 			self.library_selection.select_path((0,))
@@ -852,9 +957,9 @@ class Library(object):
 			self.libsearch_last_tooltip = new_tooltip
 			self.library.set_property('has-tooltip', False)
 			gobject.idle_add(self.library_search_tooltips_enable, widget, x, y, keyboard_mode, tooltip)
+			gobject.idle_add(widget.set_tooltip_markup, new_tooltip)
 			return 
 		
-		gobject.idle_add(widget.set_tooltip_markup, new_tooltip)
 		self.libsearch_last_tooltip = new_tooltip
 		
 		return False #api says we should return True, but this doesn't work?
@@ -932,9 +1037,7 @@ class Library(object):
 			model, rows = self.library_selection.get_selected_rows()
 		else:
 			model = self.librarydata
-			rows = []
-			for i in range(len(model)):
-				rows.append((i,))
+			rows = [(i,) for i in range(len(model))]
 		for path in rows:
 			i = model.get_iter(path)
 			pb = model.get_value(i, 0)
@@ -954,7 +1057,6 @@ class Library(object):
 							items.append(path)
 				else:
 					results, _playtime, _num_songs = self.library_return_search_items(genre=genre, artist=artist, album=album, year=year)
-					results.sort(key=lambda x: (unicode(mpdh.get(x, 'genre', 'zzz')).lower(), unicode(mpdh.get(x, 'artist', 'zzz')).lower(), unicode(mpdh.get(x, 'album', 'zzz')).lower(), mpdh.getnum(x, 'disc', '0', True, 0), mpdh.getnum(x, 'track', '0', True, 0), mpdh.get(x, 'file')))
 					for item in results:
 						items.append(mpdh.get(item, 'file'))
 		# Make sure we don't have any EXACT duplicates:
@@ -1110,13 +1212,13 @@ class Library(object):
 			return
 		self.library.freeze_child_notify()
 		currlen = len(self.librarydata)
-		bd = []
-		for item in matches:
-			if 'file' in item:
-				bd += [(self.parse_formatting(self.config.libraryformat, item, True), [self.sonatapb, self.library_set_data(path=mpdh.get(item, 'file')), self.parse_formatting(self.config.libraryformat, item, True)])]
-		bd.sort(locale.strcoll, key=operator.itemgetter(0))
-		i = 0
-		for _sort, item in bd:
+		bd = [[self.sonatapb,
+		       self.library_set_data(path=mpdh.get(item, 'file')),
+		       self.parse_formatting(self.config.libraryformat, item,
+					     True)]
+		      for item in matches if 'file' in item]
+		bd.sort(locale.strcoll, key=operator.itemgetter(2))
+		for i, item in enumerate(bd):
 			if i < currlen:
 				j = self.librarydata.get_iter((i, ))
 				for index in range(len(item)):
@@ -1124,7 +1226,6 @@ class Library(object):
 						self.librarydata.set_value(j, index, item[index])
 			else:
 				self.librarydata.append(item)
-			i += 1
 		# Remove excess items...
 		newlen = len(bd)
 		if newlen == 0:
